package net.moioli.complessi;

import org.jmlspecs.models.JMLDouble;

/**
 * Rappresentazioni alternative dei numeri complessi
 * 
 * Versione originale: 2/12/2003
 * Versione JML: 28/11/2007
 * Autore: Silvio Moioli
 */

/**
 * Rappresenta un numero complesso nella forma "a+ib".
 */
public class FormaAlgebrica implements NumeroComplesso {

	/** Parte reale del numero complesso. */
	/*@ invariant a!=Double.NaN; @*/
	private /*@ spec_public @*/ double a;
		
	/** Parte immaginaria del numero complesso. */
	/*@ invariant b!=Double.NaN; @*/
	private /*@ spec_public @*/ double b;
	
	/**
	 * Costruttore di default. Crea un nuovo numero complesso Z = (0,0).
	 */
	/*@
	  @ ensures a == 0;
	  @ ensures b == 0;
	  @*/
	public FormaAlgebrica() {
		a = 0;
		b = 0;
	}

	/**
	 * Crea un nuovo numero complesso le cui coordinate vengono passate dal
	 * chiamante.
	 * 
	 * @param a la coordinata x del numero (parte reale)
	 * @param b la coordinata y del numero (parte immaginaria)
	 */
	/*@
	  @ requires a != Double.NaN;
	  @ requires b != Double.NaN;
	  @ ensures this.a == a;
	  @ ensures this.b == b;
	  @*/
	public FormaAlgebrica(double a, double b) {
		this.a = a;
		this.b = b;
	}

	/**
	 * Costruttore di copia. Crea un nuovo numero complesso copiandolo dal
	 * numero n passato.
	 * 
	 * @param n il numero complesso da copiare
	 */
	/*@
	  @ ensures a == n.getA();
	  @ ensures b == n.getB();
	  @*/
	public FormaAlgebrica(/*@ non_null @*/ NumeroComplesso n) {
		this.a = n.getA();
		this.b = n.getB();
	}

	/**
	 * Ritorna la parte reale del numero complesso, ovvero il coefficiente "a"
	 * nella forma "a+ib". Implementa NumeroComplesso.
	 * 
	 * @return la parte reale
	 */
	public /*@ pure @*/ double getA() {
		return a;
	}

	/**
	 * Ritorna la parte immaginaria del numero complesso, ovvero il coefficiente
	 * "b" nella forma "a+ib". Implementa NumeroComplesso.
	 * 
	 * @return la parte immaginaria
	 */
	public /*@ pure @*/ double getB() {
		return b;
	}

	/**
	 * Ritorna il modulo del numero complesso, ovvero il valore dell'espressione
	 * "sqrt(a^2+b^2)" nella forma "a+ib". Implementa NumeroComplesso.
	 * 
	 * @return il modulo
	 */
	/*@
	  @ also ensures Math.abs(\result * \result - (a * a + b * b))<0.001;
      @ also ensures JMLDouble.approximatelyEqualTo(\result * \result, a * a + b * b, 0.001);
	  @*/
	public /*@ pure @*/ double getModulo() {
		return Math.sqrt(a * a + b * b);
	}

	/**
	 * Ritorna la fase del numero complesso, ovvero il valore dell'espressione
	 * "arctg(b/a)" nella forma "a+ib". Implementa NumeroComplesso.
	 * 
	 * @return la fase
	 */
	/*@
	  @ also ensures 0 <= \result && \result < 2 * Math.PI;
	  @ also ensures (0 <= \result && \result <= Math.PI / 2) <==> (a>=0 && b>=0); 
	  @*/
	public /*@ pure @*/ double getFase() {
		double result;
		if (a != 0){
			result = Math.atan(b / a);
			// Dal momento che l'arcotangente mi ritorna valori tra -PI/2 e PI/2,
			// tocca a me decidere se e' necessario aggiungere un multiplo di PI per
			// ottenere un angolo tra PI/2 e 3/2PI, a seconda delle coordinate.
			if (result > 0) {
				// a e b sono concordi, sono nel primo o terzo quadrante
				if (a > 0) {
					// primo quadrante, il risultato e' gia' corretto
				} else {
					// terzo quadrante, aggiungo PI
					result += Math.PI;
				}
			} else {
				// a e b sono discordi, sono nel secondo o quarto quadrante
				if (a > 0) {
					// quarto quadrante, aggiungo 2PI
					result += Math.PI * 2;
				} else {
					// secondo quadrante, aggiungo PI
					result += Math.PI;
				}
			}
		}
		else{//a == 0
			if(b >=0){
				result = Math.PI /2;
			}
			else{
				result = 3/2*Math.PI;
			}
		}
		
		// riduzione al primo quadrante
		while (result >= Math.PI*2){
			result -= Math.PI * 2;
		}
		while (result <= 0){
			result += Math.PI * 2;
		}

		return result;
	}

	/**
	 * Ritorna una rappresentazione in stringa del numero complesso.
	 * 
	 * @return una stringa
	 */
	public /*@ pure @*/ String toString() {
		return getA() + "+j" + getB();
	}
}
