package net.moioli.complessi;

/**
 * Rappresentazioni alternative dei numeri complessi
 * 
 * Versione originale: 2/12/2003
 * Versione JML: 28/11/2007
 * Autore: Silvio Moioli
 */

/**
 * Rappresenta un numero complesso nella forma "m*e^jf".
 */
public class FormaEsponenziale implements NumeroComplesso {
	
	/** Modulo del numero complesso. */
	/*@ invariant modulo!=Double.NaN; @*/
	private /*@ spec_public @*/ double modulo;

	/** Fase del numero complesso, in radianti. */
	/*@ invariant 0<=fase && fase< 2*Math.PI; @*/
	private /*@ spec_public @*/ double fase;

	/**
	 * Costruttore di default. Crea un nuovo numero complesso Z = (0,0).
	 */
    /*@
      @ ensures modulo == 0;
      @ ensures fase == 0;
      @*/
	public FormaEsponenziale() {
		modulo = 0;
		fase = 0;
	}

	/**
	 * Crea un nuovo numero complesso il cui modulo e la cui fase vengono
	 * specificate dall'utente.
	 * 
	 * @param modulo il modulo del numero (parte reale)
	 * @param fase la fase y del numero (parte immaginaria)
	 */
    /*@
      @ ensures this.modulo == modulo;
      @*/
	public FormaEsponenziale(double modulo, double fase) {
		this.modulo = modulo;
		// riduzione al primo quadrante
		while (fase >= Math.PI*2){
			fase -= Math.PI * 2;
		}
		while (fase <= 0){
			fase += Math.PI * 2;
		}
		this.fase = fase;
	}

	/**
	 * Costruttore di copia. Crea un nuovo numero complesso copiandolo dal
	 * numero n passato.
	 * 
	 * @param n il numero complesso da copiare
	 */
	public FormaEsponenziale(/*@ non_null @*/NumeroComplesso n) {
		this.modulo = n.getModulo();
		this.fase = n.getFase();
	}

	/**
	 * Ritorna la parte reale del numero complesso, ovvero il coefficiente "a"
	 * nella forma "a+ib". Implementa NumeroComplesso.
	 * 
	 * @return la parte reale
	 */
	public double getA() {
		return modulo * Math.cos(fase);
	}

	/**
	 * Ritorna la parte immaginaria del numero complesso, ovvero il coefficiente
	 * "b" nella forma "a+ib". Implementa NumeroComplesso.
	 * 
	 * @return la parte immaginaria
	 */
	public double getB() {
		return modulo * Math.sin(fase);
	}

	/**
	 * Ritorna il modulo del numero complesso, ovvero il valore dell'espressione
	 * "sqrt(a^2+b^2)" nella forma "a+ib". Implementa NumeroComplesso.
	 * 
	 * @return il modulo
	 */
	public double getModulo() {
		return modulo;
	}

	/**
	 * Ritorna la fase del numero complesso, ovvero il valore dell'espressione
	 * "arctg(b/a)" nella forma "a+ib". Implementa NumeroComplesso.
	 * 
	 * @return la fase
	 */
	public double getFase() {
		return fase;
	}

	/**
	 * Ritorna una rappresentazione in stringa del numero complesso.
	 * 
	 * @return una String
	 */
	public String toString() {
		return getModulo() + "e^j" + getFase();
	}
}
