package net.moioli.complessi;

/**
 * Rappresentazioni alternative dei numeri complessi
 * 
 * Versione originale: 2/12/2003
 * Versione JML: 28/11/2007
 * Autore: Silvio Moioli
 */

import java.util.Arrays;
import java.util.StringTokenizer;

/**
 * Un semplice parser di numeri complessi.
 */
public class Parser {
	/** Contiene i caratteri accettati nelle stringhe in questo parser */
	private final static byte[] legalChars = { '0', '1', '2', '3', '4', '5',
			'6', '7', '8', '9', 'j', '+', '-', '.', 'e', '^' };

	/**
	 * Controlla che la stringa passata contenga solo caratteri nell'insieme
	 * specificato da legalChars.
	 * 
	 * 
	 * @param legalChars
	 *            i caratteri ammissibili
	 * @param s
	 *            la stringa da analizzare
	 * @return true o false
	 */
	private static boolean checkChars(byte[] legalChars, String s) {
		// Preparo due array ordinati di bytes: quelli nella mia stringa...
		byte[] myStringChars = s.getBytes();
		Arrays.sort(myStringChars);

		// E quelli che potrebbero essere contenuti in essa
		Arrays.sort(legalChars);

		// Quindi, per ogni byte della mia stringa, ricerco il carattere
		// nell'insieme dei caratteri legali
		for (int i = 0; i < myStringChars.length; i++) {
			if (Arrays.binarySearch(legalChars, myStringChars[i]) < 0) {
				// Se la ricerca da' esito negativo, c'e' almeno un carattere
				// non valido
				return false;
			}
		}
		// Tutti i caratteri cercati sono legali
		return true;
	}

	/**
	 * Crea un oggetto NumeroComplesso data una stringa avente la seguente
	 * formattazione: <br>
	 * <b> "N+jM" </b> <br>
	 * dove: <br>
	 * 1- N e M sono numeri reali, e sono obbligatori. Possono essere negativi;
	 * Devono seguire la sintassi dei numeri accettati da <code> 
	 * java.util.Double.parseDouble() </code> ;
	 * <br>
	 * 2- non sono ammessi caratteri al di fuori delle cifre, '+', '-','^','.' e
	 * 'j'. <br>
	 * 
	 * @param toParse
	 *            la stringa da analizzare
	 * @return il nuovo NumeroComplesso creato
	 * @throws ParseException
	 *             in caso di errore di parsing
	 */
	private static NumeroComplesso parseFormaAlgebrica(String toParse)
			throws ParseException {
		double a = 0;
		double b = 0;

		if (checkChars(legalChars, toParse) == false) {
			throw new ParseException();
		}

		try {
			// Spezzo appena leggo 'j'
			StringTokenizer st = new StringTokenizer(toParse, "j");

			// Primo token: parte reale
			if (st.hasMoreTokens() == false) {
				throw new ParseException();
			}
			// Tolgo il '+' dopo la 'j'
			String aString = st.nextToken();
			aString = aString.substring(0, aString.length() - 1);
			a = Double.parseDouble(aString);

			// Secondo token: parte immaginaria
			if (st.hasMoreTokens() == false) {
				throw new ParseException();
			}
			// salto i caratteri "+j"
			String bString = toParse.substring(toParse.indexOf("j") + 1);
			b = Double.parseDouble(bString);
		} catch (Exception e) {// problemi nella conversione, rilancio
			throw new ParseException();
		}

		return new FormaAlgebrica(a, b);
	}

	/**
	 * Crea un oggetto NumeroComplesso data una stringa avente la seguente
	 * formattazione: <br>
	 * <b> "Ne^jM" </b> <br>
	 * dove: <br>
	 * 1- N e M sono numeri reali, e sono obbligatori. Possono essere negativi;
	 * Devono seguire la sintassi dei numeri accettati da <code> 
	 * java.util.Double.parseDouble() </code> ;
	 * <br>
	 * 2- non sono ammessi caratteri al di fuori delle cifre, '+', '-','^', '.',
	 * 'j' ed 'e'. <br>
	 * 
	 * @param toParse
	 *            la stringa da analizzare
	 * @return il nuovo NumeroComplesso creato
	 * @throws ParseException
	 *             in caso di errore di parsing
	 */
	private static NumeroComplesso parseFormaEsponenziale(String toParse)
			throws ParseException {
		double m = 0;
		double f = 0;

		try {
			// Spezzo appena leggo 'e'
			StringTokenizer st = new StringTokenizer(toParse, "e");

			// Primo token: modulo
			if (st.hasMoreTokens() == false) {
				throw new ParseException();
			}
			String mString = st.nextToken();
			m = Double.parseDouble(mString);

			if (m < 0) {
				throw new ParseException();// il modulo deve essere positivo!
			}

			// Secondo token: fase
			if (st.hasMoreTokens() == false) {
				throw new ParseException();
			}
			// salto i caratteri "e^j"
			String fString = toParse.substring(toParse.indexOf("e") + 3);
			f = Double.parseDouble(fString);
		} catch (Exception e) {// problemi nella conversione, rilancio
			throw new ParseException();
		}

		return new FormaEsponenziale(m, f);
	}

	/**
	 * Crea un oggetto NumeroComplesso data una stringa avente una delle
	 * seguenti formattazioni: <br>
	 * <b> "N+jM" </b> <br>
	 * oppure <br>
	 * <b> "Ne^jM" </b> <br>
	 * dove: <br>
	 * 1- N e M sono numeri reali, e sono obbligatori. Possono essere negativi;
	 * Devono seguire la sintassi dei numeri accettati da <code> 
	 * java.util.Double.parseDouble() </code> ;
	 * <br>
	 * 2- non sono ammessi caratteri al di fuori delle cifre, '+', '-', '.', 'j'
	 * ed 'e'. <br>
	 * 
	 * @param toParse
	 *            la stringa da analizzare
	 * @return il nuovo NumeroComplesso creato
	 * @throws ParseException
	 *             in caso di errore di parsing
	 */
	public static NumeroComplesso parse(String toParse) throws ParseException {
		if (checkChars(legalChars, toParse) == false) {
			throw new ParseException();
		}
		if (isAlgebrica(toParse)) {// il numero e' in forma algebrica
			return parseFormaAlgebrica(toParse);
		} else {// forma esponenziale
			return parseFormaEsponenziale(toParse);
		}
	}

	/**
	 * Determina se una stringa rappresenta un numero in forma algebrica.
	 * 
	 * @param la
	 *            stringa
	 * @return true o false
	 */
	public static boolean isAlgebrica(String s) {
		return s.indexOf("e") == -1;
	}
}