package net.moioli.grafici;

/**
 * Rappresentazioni alternative dei numeri complessi
 * 
 * Versione originale: 2/12/2003
 * Versione JML: 28/11/2007
 * Autore: Silvio Moioli
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;

/**
 * Rappresenta una coppia di assi coordinati di un grafico, le rispettive unita'
 * di misura, e la legenda associata.
 */
public class AssiCartesiani implements Disegnabile {
	/**
	 * Ampiezza dell'angolo delle freccette degli assi
	 */
	private static final double ANGOLOFRECCIA = Math.PI / 6;
	/**
	 * Altera le dimensioni dei dettagli (freccette, lineette). Calcolato come
	 * risoluzioneX/GRANDEZZADETTAGLI.
	 */
	private final static double GRANDEZZADETTAGLI = 50;

	/**
	 * Calcola la distanza in pixels tra una tacchetta e un'altra (asse delle x)
	 * 
	 * @return la distanza (pixels) tra una tacchetta e un'altra
	 */
	public int larghezzaIntervalloX(Scala s) {
		/*
		 * Se fossero richieste troppe "tacchette", ne disegno meno ma cambio
		 * scala. Se sono buono :)
		 */

		if ((s.getZoomX() > (s.getScreenMaxX() / 20)) && (s.getZoomX() > 2)) {
			return (int) Math.round(s.getZoomX());
		} else {
			if (Math.round(s.getScreenMaxX() / 10) < 1) {
				return 1;
			} else {
				return (int) Math.round(s.getScreenMaxX() / 10);
			}
		}
	}

	/**
	 * Calcola la distanza in pixels tra una tacchetta e un'altra (asse delle y)
	 * 
	 * @return la distanza (pixels) tra una tacchetta e un'altra
	 */
	public int larghezzaIntervalloY(Scala s) {
		/*
		 * Se fossero richieste troppe "tacchette", ne disegno meno ma cambio
		 * scala. Se sono buono :)
		 */

		if ((s.getZoomY() > (s.getScreenMaxY() / 20)) && (s.getZoomY() > 2)) {
			return (int) Math.round(s.getZoomY());
		} else {
			if (Math.round(s.getScreenMaxY() / 10) < 1) {
				return 1;
			} else {
				return (int) Math.round(s.getScreenMaxY() / 10);
			}
		}
	}

	/**
	 * Calcola la grandezza rappresentata da una tacchetta sul grafico (asse
	 * delle x)
	 * 
	 * @return la grandezza rappresentata da una tacchetta sul grafico
	 * 
	 */
	public double valoreIntervalloX(Scala s) {
		if ((s.getZoomX() > (s.getScreenMaxX() / 20)) && (s.getZoomX() > 2)) {
			return 1;
		} else {
			return (s.getScreenMaxX() / 10) / s.getZoomX();
		}
	}

	/**
	 * Calcola la grandezza rappresentata da una tacchetta sul grafico (asse
	 * delle y)
	 * 
	 * @return la grandezza rappresentata da una tacchetta sul grafico
	 * 
	 */
	public double valoreIntervalloY(Scala s) {
		if ((s.getZoomY() > (s.getScreenMaxY() / 20)) && (s.getZoomY() > 2)) {
			return 1;
		} else {
			return (s.getScreenMaxY() / 10) / s.getZoomY();
		}
	}

	/**
	 * Calcola la grandezza dei "dettagli" come le freccette o le lineette sugli
	 * assi.
	 * 
	 * @return la grandezza rappresentata da una tacchetta sul grafico
	 * 
	 */
	public int getDettagli(Scala s) {
		return (int) Math.round(s.getScreenMaxX() / GRANDEZZADETTAGLI);
	}

	/**
	 * Implementa l'interfaccia Disegnabile, disegnando questo oggetto.
	 * 
	 * @param g
	 *            l'oggetto di grafica sul quale disegnare
	 * @param s
	 *            la scala relativa al grafico e all'oggetto Graphics
	 * @param c
	 *            il colore con cui disegnare questo oggetto
	 */
	public void disegnami(Graphics g, Scala s, Color c) {
		g.setColor(c);

		// Calcolo le larghezze delle "tacchette" sul grafico
		int larghX = larghezzaIntervalloX(s);
		int larghY = larghezzaIntervalloY(s);

		// Calcolo quanto rappresenta una "tacchetta" del grafico
		double xInUno = valoreIntervalloX(s);
		double yInUno = valoreIntervalloY(s);

		// calcolo i punti per disegnare gli assi
		//
		// C.
		// ^
		// |
		// |
		// |
		// A.--------------------------------->.B
		// |O
		// |
		// |
		// |
		// |
		// D.
		//
		Point A = new Point(s.calcolaX(s.getMinX()), s.calcolaY(0));

		Point B = new Point(s.calcolaX(s.getMaxX()), s.calcolaY(0));

		Point C = new Point(s.calcolaX(0), s.calcolaY(s.getMaxY()));

		Point D = new Point(s.calcolaX(0), s.calcolaY(s.getMinY()));

		// disegno gli assi
		g.drawLine(A.x, A.y, B.x, B.y);
		g.drawLine(C.x, C.y, D.x, D.y);

		// calcolo i punti per le frecce
		/*
		 * Point Bf1 = new Point((int)Math.round(B.x - getDettagli(s)),
		 * (int)Math.round(B.y - getDettagli(s))); Point Bf2 = new
		 * Point((int)Math.round(B.x - getDettagli(s)), (int)Math.round(B.y +
		 * getDettagli(s))); Point Cf1 = new Point((int)Math.round(C.x -
		 * getDettagli(s)), (int)Math.round(C.y + getDettagli(s))); Point Cf2 =
		 * new Point((int)Math.round(C.x + getDettagli(s)), (int)Math.round(C.y +
		 * getDettagli(s)));
		 */

		// disegno le frecce
		disegnaFreccia(A, B, g, s);
		disegnaFreccia(D, C, g, s);

		// disegno le lineette delle unita'
		for (int i = C.x; i < s.getScreenMaxX(); i += larghX) {
			g.drawLine(i, (int) Math.round(A.y - getDettagli(s)), i, (int) Math
					.round(A.y + getDettagli(s)));
		}
		for (int i = C.x; i > 0; i -= larghX) {
			g.drawLine(i, (int) Math.round(A.y - getDettagli(s)), i, (int) Math
					.round(A.y + getDettagli(s)));
		}
		for (int i = A.y; i < s.getScreenMaxY(); i += larghY) {
			g.drawLine((int) Math.round(C.x - getDettagli(s)), i, (int) Math
					.round(C.x + getDettagli(s)), i);
		}
		for (int i = A.y; i > 0; i -= larghY) {
			g.drawLine((int) Math.round(C.x - getDettagli(s)), i, (int) Math
					.round(C.x + getDettagli(s)), i);
		}

		// metto le lettere agli assi coordinati
		g.drawString("x", B.x - 10, B.y + 10);
		g.drawString("y", C.x - 10, C.y + 10);

		// disegno la legenda
		ElementoLegenda e[] = { new ElementoLegenda("X:", larghX, xInUno),
				new ElementoLegenda("Y:", larghY, yInUno) };
		Legenda leg = new Legenda(e);
		leg.disegnami(g, s, c);
	}

	private void disegnaFreccia(Point pFrom, Point pTo, Graphics g, Scala s) {
		// Si ringraziano i gentili autori di BlueJ per NON averlo reso
		// opensource...
		// Tanto il codice si legge lo stesso ;)
		double angle = Math.atan2(-(pFrom.getY() - pTo.getY()), pFrom.getX()
				- pTo.getX());
		int xPoints[] = {
				pTo.x,
				pTo.x
						+ (int) (2 * getDettagli(s) * Math.cos(angle
								+ ANGOLOFRECCIA)),
				pTo.x
						+ (int) (2 * getDettagli(s) * Math.cos(angle
								- ANGOLOFRECCIA)) };
		int yPoints[] = {
				pTo.y,
				pTo.y
						- (int) (2 * getDettagli(s) * Math.sin(angle
								+ ANGOLOFRECCIA)),
				pTo.y
						- (int) (2 * getDettagli(s) * Math.sin(angle
								- ANGOLOFRECCIA)) };
		g.fillPolygon(xPoints, yPoints, 3);
	}

	/**
	 * Dal momento che gli assi vengono comunque disegnati, questa
	 * implementazione ritorna sempre NaN.
	 * 
	 * @param minX
	 *            il minimo valore reale delle x dell'intervallo
	 * @param maxX
	 *            il massimo valore reale delle x dell'intervallo
	 * @return il massimo valore delle ordinate o NaN
	 */
	public double maxY(double minX, double maxX) {
		return Double.NaN;
	}

	/**
	 * Dal momento che gli assi vengono comunque disegnati, questa
	 * implementazione ritorna sempre NaN.
	 * 
	 * @param minX
	 *            il minimo valore reale delle x dell'intervallo
	 * @param maxX
	 *            il massimo valore reale delle x dell'intervallo
	 * @return il massimo valore delle ordinate o NaN
	 */
	public double minY(double minX, double maxX) {
		return Double.NaN;
	}
}
