package net.moioli.grafici;

/**
 * Rappresentazioni alternative dei numeri complessi
 * 
 * Versione originale: 2/12/2003
 * Versione JML: 28/11/2007
 * Autore: Silvio Moioli
 */

import java.awt.Color;
import java.awt.Graphics;

/**
 * Rappresenta un elemento della legenda di un grafico (cartesiano).
 */
class ElementoLegenda implements Disegnabile {
	/** Lunghezza in pixel del tratto di questa legenda */
	private int lunghezza = 10;
	/** Valore reale del tratto di questa legenda */
	private double valore = 1;
	/** Eventuale altra descrizione */
	private String descrizione = "";
	/** Posizione relativa di questo elemento (x) */
	private int x = 0;
	/** Posizione relativa di questo elemento (y) */
	private int y = 0;

	/**
	 * Costruttore di default
	 */
	public ElementoLegenda() {
		descrizione = "";
		lunghezza = 10;
		valore = 1;
		x = 0;
		y = 0;
	}

	/**
	 * Costruttore standard
	 * 
	 * @param descrizione
	 *            Una stringa descrittiva dell'elemento
	 * @param lunghezza
	 *            La lunghezza (in pixels) dell'elemento (cioe' questo |---|)
	 * @param valore
	 *            Il valore numerico dell'elemento (a quanto corrisponde una
	 *            tacchetta)
	 * 
	 */
	public ElementoLegenda(String descrizione, int lunghezza, double valore) {
		this.descrizione = descrizione;
		this.lunghezza = lunghezza;
		this.valore = valore;
	}

	/**
	 * Permette di spostare questo elemento nella legenda.
	 * 
	 * @param x
	 *            la nuova posizione relativa di questo elemento
	 * @param y
	 *            la nuova posizione relativa di questo elemento
	 */
	public void setXY(int x, int y) {
		this.x = x;
		this.y = y;
	}

	/**
	 * Implementa l'interfaccia Disegnabile, disegnando questo oggetto.
	 * 
	 * @param g
	 *            l'oggetto di grafica sul quale disegnare
	 * @param s
	 *            la scala relativa al grafico e all'oggetto Graphics
	 * @param c
	 *            il colore con cui disegnare questo oggetto
	 */
	public void disegnami(Graphics g, Scala s, Color c) {
		// calcolo della grandezza dei dettagli
		int l = g.getFontMetrics().stringWidth(" ");

		// calcolo la lunghezza in pixels della stringa di descrizione
		int lDescrizione = g.getFontMetrics().stringWidth(descrizione);

		// calcolo l'altezza in pixels della stringa di descrizione
		int aDescrizione = g.getFontMetrics().getHeight();

		// Calcolo la stringa rappresentante il valore e l'offset a cui deve
		// essere
		// disegnata
		String stringValore = Double.toString(valore);
		int charsToBeDrawn = 1;
		int stringValoreWidth = g.getFontMetrics().stringWidth(
				stringValore.substring(0, charsToBeDrawn));
		while ((stringValoreWidth < lunghezza)
				&& (charsToBeDrawn < stringValore.length())) {
			charsToBeDrawn += 1;
			stringValoreWidth = g.getFontMetrics().stringWidth(
					stringValore.substring(0, charsToBeDrawn));
		}
		if (stringValore.length() > charsToBeDrawn)
			stringValore = stringValore.substring(0, charsToBeDrawn);
		int offset = (lunghezza - (g.getFontMetrics().stringWidth(stringValore))) / 2;

		// calcolo dei punti in alto a SX
		int xIniziale = x - (lDescrizione + 2 * l + lunghezza);
		int yIniziale = y + aDescrizione;

		// disegno
		g.drawString(descrizione, xIniziale, yIniziale + aDescrizione / 2);
		g.drawLine(xIniziale + lDescrizione, yIniziale, xIniziale
				+ lDescrizione, yIniziale + l);
		g.drawLine(xIniziale + lDescrizione + lunghezza, yIniziale, xIniziale
				+ lDescrizione + lunghezza, yIniziale + l);
		g.drawLine(xIniziale + lDescrizione, yIniziale + l / 2, xIniziale
				+ lDescrizione + lunghezza, yIniziale + l / 2);

		g
				.drawString(stringValore, xIniziale + lDescrizione + offset,
						yIniziale);
	}

	/**
	 * Ritorna l'altezza (in pixel) dell'elemento una volta disegnato.
	 * 
	 * @param g
	 *            L'oggeto Graphics2D su cui si intende disegnare l'oggetto
	 */
	public int getHeight(Graphics g) {
		return g.getFontMetrics().getHeight() * 2;
	}

	/**
	 * Dal momento che la legenda viene comunque disegnata, questa
	 * implementazione ritorna sempre NaN.
	 * 
	 * @param minX
	 *            il minimo valore reale delle x dell'intervallo
	 * @param maxX
	 *            il massimo valore reale delle x dell'intervallo
	 * @return il massimo valore delle ordinate o NaN
	 */
	public double maxY(double minX, double maxX) {
		return Double.NaN;
	}

	/**
	 * Dal momento che la legenda viene comunque disegnata, questa
	 * implementazione ritorna sempre NaN.
	 * 
	 * @param minX
	 *            il minimo valore reale delle x dell'intervallo
	 * @param maxX
	 *            il massimo valore reale delle x dell'intervallo
	 * @return il massimo valore delle ordinate o NaN
	 */
	public double minY(double minX, double maxX) {
		return Double.NaN;
	}
}
