package net.moioli.grafici;

/**
 * Rappresentazioni alternative dei numeri complessi
 * 
 * Versione originale: 2/12/2003
 * Versione JML: 28/11/2007
 * Autore: Silvio Moioli
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;

/**
 * Decorator per CompatibleCollection, permette di disegnare una collezione di
 * oggetti disegnabili.
 */
public class Grafico implements CompatibleCollection, IGrafico {
	/**
	 * Indica ogni quanti pixel sara' ricalcolato il valore della funzione.
	 * Minimo 1.
	 */
	/*@ private invariant 0<precisionePixel && precisionePixel<=10; @*/
	private int precisionePixel = 1;

	/** Indica il minimo valore della x rappresentato */
	private double minX = -10;
	
	/** Indica il massimo valore della x rappresentato */
	/*@ private invariant maxX>minX; @*/
	private double maxX = 10;

	/**
	 * Indica il minimo valore della y rappresentato. NaN per
	 * autodeterminazione.
	 */
	private double minY = Double.NaN;

	/**
	 * Indica il massimo valore della y rappresentato. NaN per
	 * autodeterminazione.
	 */
	/*@ private invariant Double.isNaN(maxY) || maxY>minY; @*/
	private double maxY = Double.NaN;

	/** Collezione interna degli oggetti da disegnare. */
	/*@ invariant c != null; @*/
	public /*@ non_null @*/ CompatibleCollection c = new CollectionWrapper(new LinkedList());
	
	/** Generatore comune a tutti i Grafici */
	private static /*@ non_null @*/ Random generator = new Random();
	
	/*
	 * Crea un colore pseudocasuale. Il range di valori per R, G e B va da 0 a
	 * 255, ma e' limitato a 0 a 127 in modo da ottenere toni scuri piu' leggibili
	 * su fondo chiaro.
	 */
	/** Colore con cui sono disegnati gli oggetti in questo Grafico. */
	private /*@ non_null @*/ Color color = new Color(generator.nextInt(128), generator
			.nextInt(128), generator.nextInt(128));

	/**
	 * Costruisce un grafico vuoto.
	 */
	public Grafico() {
		setPrecisionePixel(1);
		setMinX(-10);
		setMaxX(10);
		/*
		 * setMinY(-10); setMaxY(10);
		 */
		add(new AssiCartesiani());
	}

	/**
	 * Costruisce un grafico con piu' oggetti.
	 * 
	 * @param c Collection di Disegnabile da rappresentare
	 */
	public Grafico(/*@ non_null @*/CompatibleCollection c) {
		setPrecisionePixel(1);
		setMinX(-10);
		setMaxX(10);
		Collection newC = new LinkedList();
		newC.addAll(new CompatibleCollectionWrapper(c));
		newC.add(new AssiCartesiani());
		addAll(newC);
	}

	/**
	 * Disegna il grafico sull'oggetto Graphics passato
	 * 
	 * @param g l'oggetto su cui disegnare
	 */
	public void disegnami(/*@ non_null @*/ Graphics g, /*@ non_null @*/ Rectangle clip) {
		// Imposta i settaggi per il rendering
		//@ assert g instanceof Graphics2D;
		Graphics2D g2d = (Graphics2D) g;
		RenderingHints rh = g2d.getRenderingHints();
		rh.put(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		rh.put(RenderingHints.KEY_TEXT_ANTIALIASING,
				RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		g2d.setRenderingHints(rh);

		// Calcola la scala
		Scala s = null;
		// Se minY e maxY sono settati, vengono passati automaticamente a Scala
		// (risparmiando sul tempo di verifica delle funzioni)
		if (Double.isNaN(getMaxY()) || Double.isNaN(getMinY())) {
			setMinY(Double.NaN);
			setMaxY(Double.NaN);
			s = new Scala(this, clip, getMinX(), getMaxX(), precisionePixel);
		} else {
			s = new Scala(clip, getMinX(), getMaxX(), getMinY(), getMaxY(),
					precisionePixel);
		}

		// Disegno
		Iterator i = this.iterator();

		// Assi in nero
		((Disegnabile) i.next()).disegnami(g, s, Color.BLACK);

		while (i.hasNext()) {
			Object o = i.next();
			//@ assert o instanceof Disegnabile;
			((Disegnabile) o).disegnami(g, s, color);
		}
	}

	// Getters and Setters

	/**
	 * @return Returns the color.
	 */
	public Color getColor() {
		return color;
	}

	/**
	 * @param color
	 *            The color to set.
	 */
	public void setColor(/*@ non_null @*/ Color color) {
		this.color = color;
	}

	/**
	 * @return Returns the maxX.
	 */
	public double getMaxX() {
		return maxX;
	}

	/**
	 * @param maxX
	 *            The maxX to set.
	 */
	public void setMaxX(double maxX) {
		this.maxX = maxX;
	}

	/**
	 * @return Returns the minX.
	 */
	public double getMinX() {
		return minX;
	}

	/**
	 * @param minX
	 *            The minX to set.
	 */
	public void setMinX(double minX) {
		this.minX = minX;
	}

	/**
	 * @return Returns the maxY.
	 */
	public double getMaxY() {
		return maxY;
	}

	/**
	 * @param maxY
	 *            The maxY to set.
	 */
	public void setMaxY(double maxY) {
		this.maxY = maxY;
	}

	/**
	 * @return Returns the minY.
	 */
	public double getMinY() {
		return minY;
	}

	/**
	 * @param minY
	 *            The minY to set.
	 */
	public void setMinY(double minY) {
		this.minY = minY;
	}

	/**
	 * @return Returns the precisionePixel.
	 */
	public int getPrecisionePixel() {
		return precisionePixel;
	}

	/**
	 * @param precisionePixel
	 *            The precisionePixel to set.
	 */
	public void setPrecisionePixel(int precisionePixel) {
		this.precisionePixel = precisionePixel;
	}

	/*@ 
	  @ also requires !this.contains(arg0);
	  @ also ensures this.contains(arg0);
	  @ also ensures c.getCollectionSize() == \old(c.getCollectionSize()) + 1;
	  @*/
	public boolean add(/*@ non_null @*/ Object arg0) {
		return c.add(arg0);
	}

	/*@ 
	  @ also requires (\forall Object x; arg0.contains(x); !this.contains(x));
	  @ also ensures (\forall  Object x; arg0.contains(x); this.contains(x));
	  @*/
	public boolean addAll(/*@ non_null @*/ Collection arg0) {
		return c.addAll(arg0);
	}

	/**
	 * 
	 */
	public void clear() {
		c.clear();
		c.add(new AssiCartesiani());
	}

	/**
	 * @param arg0
	 * @return
	 */
	public /*@ pure @*/ boolean contains(Object arg0) {
		return c.contains(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 */
	public boolean containsAll(Collection arg0) {
		return c.containsAll(arg0);
	}

	/*
	 * (non Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object arg0) {
		return c.equals(arg0);
	}

	/*
	 * (non Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return c.hashCode();
	}

	/**
	 * @return
	 */
	public boolean isEmpty() {
		return c.isEmpty();
	}

	/**
	 * @return
	 */
	public Iterator iterator() {
		return c.iterator();
	}

	/**
	 * @param arg0
	 * @return
	 */
	public boolean remove(Object arg0) {
		return c.remove(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 */
	public boolean removeAll(Collection arg0) {
		return c.removeAll(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 */
	public boolean retainAll(Collection arg0) {
		return c.retainAll(arg0);
	}

	/**
	 * @return
	 */
	public /*@ pure @*/ int getCollectionSize() {
		return c.getCollectionSize();
	}

	/**
	 * @return
	 */
	public Object[] toArray() {
		return c.toArray();
	}

	/**
	 * @param arg0
	 * @return
	 */
	public Object[] toArray(Object[] arg0) {
		return c.toArray(arg0);
	}

	/*
	 * (non Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return c.toString();
	}
}
