package net.moioli.grafici;

/**
 * Rappresentazioni alternative dei numeri complessi
 * 
 * Versione originale: 2/12/2003
 * Versione JML: 28/11/2007
 * Autore: Silvio Moioli
 */

import java.awt.Rectangle;
import java.util.Iterator;

/**
 * Raccoglie alcuni dei dati necessari al calcolo della scala del grafico.
 */
public class Scala {
	/** Minimo valore rappresentato sull'asse delle x */
	private double minX = -10;
	/** Massimo valore rappresentato sull'asse delle x */
	private double maxX = 10;
	/** Minimo valore rappresentato sull'asse delle y */
	private double minY = -1;
	/** Massimo valore rappresentato sull'asse delle y */
	private double maxY = 1;
	/** Massimo numero di pixels disponibili (orizzontali) */
	private int screenMaxX = 100;
	/** Massimo numero di pixels disponibili (verticali) */
	private int screenMaxY = 100;
	/**
	 * Indica ogni quanti pixels sara' ricalcolato il valore della funzione.
	 * Minimo 1.
	 */
	private int precisionePixel = 3;

	/**
	 * Massimo valore rappresentabile nel grafico usato quando una o piu'
	 * funzioni restituiscono un valore infinito.
	 */
	private static final double VALORETAGLIO = 100000;

	/**
	 * Costruttore di default (per compatibilita' beans).
	 */
	Scala() {
	}

	/**
	 * Costruttore standard.
	 * 
	 * @param c la collezione di oggetti da disegnare (implementanti
	 *            IDisegnabile)
	 * @param clip il rettangolo su cui disegnare
	 * @param minX il minimo valore da rappresentare sulle x
	 * @param maxX il massimo valore da rappresentare sulle x
	 * @param minY il minimo valore da rappresentare sulle y
	 * @param maxY il massimo valore da rappresentare sulle y
	 * @param precisionePixe il numero massimo di pixels interpolati (1=migliore)
	 */
	Scala(Rectangle clip, double minX, double maxX, double minY, double maxY,
			int precisionePixel) {
		// Copio i valori minimi e massimi delle x e delle y
		setMinX(minX);
		setMaxX(maxX);
		setMinY(minY);
		setMaxY(maxY);

		// Calcolo i pixels disponibili
		setScreenMaxX((int) clip.getWidth() - 1);
		setScreenMaxY((int) clip.getHeight() - 1);

		// Setto la precisione
		setPrecisionePixel(precisionePixel);
	}

	/**
	 * Costruttore con rilevamento delle y. Calcola automaticamente i valori di
	 * minY e maxY, che pertanto sono omessi.
	 * 
	 * @param c la collezione di oggetti da disegnare (implementanti
	 * IDisegnabile)
	 * @param clip il rettangolo su cui disegnare
	 * @param minX il minimo valore da rappresentare sulle x
	 * @param maxX il massimo valore da rappresentare sulle x
	 * @param precisionePixel il numero massimo di pixels interpolati (1=migliore)
	 */
	Scala(CompatibleCollection c, Rectangle clip, double minX, double maxX,
			int precisionePixel) {
		this(clip, minX, maxX, -1, 1, precisionePixel);
		// Calcolo i valori minimi e massimi delle y
		Iterator i = c.iterator();
		while (i.hasNext()) {
			Disegnabile d = (Disegnabile) i.next();
			setNewY(d.minY(minX, maxX));
			setNewY(d.maxY(minX, maxX));
		}
	}

	/**
	 * Calcola l'ascissa in pixel rispetto allo schermo calcolata in base
	 * all'ascissa del punto reale passato.
	 * 
	 * @param x ascissa del punto che deve essere disegnato
	 * @return il numero di pixels dall'angolo in alto a sinistra dello schermo
	 */
	public int calcolaX(double x) {
		int result = (int) Math.round(getZoomX() * (x - getMinX()));
		if (result > getScreenMaxX()) {
			return getScreenMaxX();
		} else {
			if (result < 0) {
				return 0;
			} else {
				return result;
			}
		}
	}

	/**
	 * Calcola l'ordinata in pixel rispetto allo schermo calcolata in base
	 * all'ordinata del punto reale passato.
	 * 
	 * @param y ordinata del punto che deve essere disegnato
	 * @return il numero di pixels dall'angolo in alto a sinistra dello schermo
	 */
	public int calcolaY(double y) {
		int result = (int) Math.round(getScreenMaxY()
				- (getZoomY() * (y - getMinY())));
		if (result > getScreenMaxY()) {
			return getScreenMaxY();
		} else {
			if (result < 0) {
				return 0;
			} else {
				return result;
			}
		}
	}

	/**
	 * Calcola il valore dell'incremento da dare alle x per calcolare la f(x) in
	 * modo che sia rappresentata con la precisione specificata da
	 * precisionePixel su questo oggetto Graphics.
	 * 
	 * @return l'incremento delle x
	 */
	public double getPasso() {
		return getPrecisionePixel() / getZoomX();
	}

	/**
	 * Calcola il valore dello zoom sulle x.
	 * 
	 * @return zoom sulle x
	 */
	public double getZoomX() {
		return getScreenMaxX() / (getMaxX() - getMinX());
	}

	/**
	 * Calcola il valore dello zoom sulle y.
	 * 
	 * @return zoom sulle y
	 */
	public double getZoomY() {
		return getScreenMaxY() / (getMaxY() - getMinY());
	}

	/**
	 * Calcola il numero di pixels della lunghezza di un segmento parallelo
	 * all'asse x dato.
	 * 
	 * @param l
	 *            la lunghezza da calcolare
	 */
	public int calcolaLunghezzaX(double l) {
		return (int) Math.round(l * getZoomX());
	}

	/**
	 * Calcola il numero di pixels della lunghezza di un segmento parallelo
	 * all'asse y dato.
	 * 
	 * @param l
	 *            la lunghezza da calcolare
	 */
	public int calcolaLunghezzaY(double l) {
		return (int) Math.round(l * getZoomY());
	}

	/**
	 * Fa in modo che la scala comprenda sicuramente il punto di ordinata
	 * specificata. E' sufficiente chiamare questo metodo per ogni punto che
	 * deve assolutamente essere disegnato sul grafico. Alternativamente si
	 * possono fare due chiamate a questo metodo (una per il minimo e una il
	 * massimo).
	 * 
	 * @param y
	 *            ascissa del punto che deve essere disegnato sicuramente.
	 */
	public void setNewY(double y) {
		if (y > getMaxY()) {
			setMaxY(y);
		}
		if (y < getMinY()) {
			setMinY(y);
		}

		if (getMinY() < -VALORETAGLIO) {
			setMinY(-VALORETAGLIO);
		}
		if (getMaxY() > VALORETAGLIO) {
			setMaxY(VALORETAGLIO);
		}
	}

	// Getters and Setters
	/**
	 * @return Returns the maxX.
	 */
	public double getMaxX() {
		return maxX;
	}

	/**
	 * @param maxX
	 *            The maxX to set.
	 */
	public void setMaxX(double maxX) {
		this.maxX = maxX;
	}

	/**
	 * @return Returns the maxY.
	 */
	public double getMaxY() {
		return maxY;
	}

	/**
	 * @param maxY
	 *            The maxY to set.
	 */
	public void setMaxY(double maxY) {
		this.maxY = maxY;
	}

	/**
	 * @return Returns the minX.
	 */
	public double getMinX() {
		return minX;
	}

	/**
	 * @param minX
	 *            The minX to set.
	 */
	public void setMinX(double minX) {
		this.minX = minX;
	}

	/**
	 * @return Returns the minY.
	 */
	public double getMinY() {
		return minY;
	}

	/**
	 * @param minY
	 *            The minY to set.
	 */
	public void setMinY(double minY) {
		this.minY = minY;
	}

	/**
	 * @return Returns the precisionePixel.
	 */
	public int getPrecisionePixel() {
		return precisionePixel;
	}

	/**
	 * @param precisionePixel
	 *            The precisionePixel to set.
	 */
	public void setPrecisionePixel(int precisionePixel) {
		this.precisionePixel = precisionePixel;
	}

	/**
	 * @return Returns the screenMaxX.
	 */
	public int getScreenMaxX() {
		return screenMaxX;
	}

	/**
	 * @param screenMaxX
	 *            The screenMaxX to set.
	 */
	public void setScreenMaxX(int screenMaxX) {
		this.screenMaxX = screenMaxX;
	}

	/**
	 * @return Returns the screenMaxY.
	 */
	public int getScreenMaxY() {
		return screenMaxY;
	}

	/**
	 * @param screenMaxY
	 *            The screenMaxY to set.
	 */
	public void setScreenMaxY(int screenMaxY) {
		this.screenMaxY = screenMaxY;
	}
}
