package net.moioli.grafici;

/**
 * Rappresentazioni alternative dei numeri complessi
 * 
 * Versione originale: 2/12/2003
 * Versione JML: 28/11/2007
 * Autore: Silvio Moioli
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;

import net.moioli.complessi.NumeroComplesso;

/**
 * Un oggetto disegnabile su un Grafico rappresentante un vettore.
 */
public class VettoreDisegnabile implements Disegnabile {
	/** Ascissa del secondo punto che rappresenta il vettore */
	private double x = 0;
	/** Ordinata del secondo punto che rappresenta il vettore */
	private double y = 0;
	/** Ampiezza dell'angolo della freccietta */
	private static final double ANGOLOFRECCIA = Math.PI / 6;
	/** Dimensioni dei dettagli */
	private static final int DETTAGLI = 20;

	/**
	 * Costruttore standard.
	 * 
	 * @param x
	 *            ascissa del secondo punto che rappresenta il vettore
	 * @param y
	 *            ordinata del secondo punto che rappresenta il vettore
	 */
	public VettoreDisegnabile(double x, double y) {
		this.x = x;
		this.y = y;
	}

	/**
	 * Costruttore tramite numero complesso
	 * 
	 * @param z
	 *            il numero complesso da cui ricavare il vettore
	 */
	public VettoreDisegnabile(NumeroComplesso z) {
		this(z.getA(), z.getB());
	}

	/**
	 * Implementa Disegnabile. L'utente non dovrebbe mai richiamare questo
	 * metodo, visto che e' richiamato automaticamente dall'oggetto Grafico.
	 * All'interno di questo metodo dovrebbe esserci tutto il codice necessario
	 * al tracciamento sul video di quest'oggetto. Tutte le operaizoni di scala
	 * dovrebbero essere fatte esclusivamente tramite l'oggetto ScalaGrafico
	 * passato.
	 * 
	 * @param g
	 *            la superficie dove questo oggetto verra' disegnato
	 * @param s
	 *            l'oggetto Scala da utilizzare per il tracciamento dell'oggetto
	 * @param c
	 *            il colore con cui disegnare l'oggetto
	 */
	public void disegnami(Graphics g, Scala s, Color c) {
		// calcolo i due punti per disegnare il vettore
		// origine
		int x1 = s.calcolaX(0);
		int y1 = s.calcolaY(0);

		// punto finale (testa della freccia)
		int x2 = s.calcolaX(x);
		int y2 = s.calcolaY(y);
		disegnaFreccia(x1, y1, x2, y2, (int) DETTAGLI, g);
	}

	/**
	 * Disegna una freccia su un Grafico.
	 * 
	 * @param x1
	 *            ascissa del primo punto
	 * @param y1
	 *            ordinata del primo punto
	 * @param x2
	 *            ascissa del secondo punto (qui ci sara' la punta della freccia)
	 * @param y2
	 *            ascissa del secondo punto (qui ci sara' la punta della freccia)
	 * @param dettagli
	 *            la grandezza dei dettagli in pixels
	 * @param g
	 *            l'oggetto di grafica su cui disegnare
	 */
	private void disegnaFreccia(int x1, int y1, int x2, int y2, int dettagli,
			Graphics g) {
		// Si ringraziano i gentili autori di BlueJ per NON averlo reso
		// opensource...
		// Tanto il codice si legge lo stesso ;)
		Point pFrom = new Point(x1, y1);
		Point pTo = new Point(x2, y2);
		double angle = Math.atan2(-(pFrom.getY() - pTo.getY()), pFrom.getX()
				- pTo.getX());
		int xPoints[] = { pTo.x,
				pTo.x + (int) (dettagli * Math.cos(angle + ANGOLOFRECCIA)),
				pTo.x + (int) (dettagli * Math.cos(angle - ANGOLOFRECCIA)) };
		int yPoints[] = { pTo.y,
				pTo.y - (int) (dettagli * Math.sin(angle + ANGOLOFRECCIA)),
				pTo.y - (int) (dettagli * Math.sin(angle - ANGOLOFRECCIA)) };
		g.drawPolygon(xPoints, yPoints, 3);
		g.drawLine(pFrom.x, pFrom.y, pTo.x, pTo.y);
	}

	/**
	 * Ritorna la minima ordinata dell'oggetto da rappresentare se ne fosse
	 * disegnata la porzione compresa fra un'ascissa minima e una massima.<br>
	 * 
	 * Se l'oggetto in questione non ha una ordinata massima o non ha senso
	 * definirne una, questa funzione deve ritornare il valore Double.NaN.
	 * 
	 * @param minX
	 *            il minimo valore reale delle x dell'intervallo
	 * @param maxX
	 *            il massimo valore reale delle x dell'intervallo
	 * @return il minimo valore delle ordinate o NaN
	 */
	public double minY(double minX, double maxX) {
		if ((x >= minX) && (x <= maxX)) {
			return y;
		} else {
			return java.lang.Double.NaN;
		}
	}

	/**
	 * Ritorna la massima ordinata dell'oggetto da rappresentare se ne fosse
	 * disegnata la porzione compresa fra un'ascissa minima e una massima.<br>
	 * 
	 * Se l'oggetto in questione non ha una ordinata minima o non ha senso
	 * definirne una, questa funzione deve ritornare il valore Double.NaN.
	 * 
	 * @param minX
	 *            il minimo valore reale delle x dell'intervallo
	 * @param maxX
	 *            il massimo valore reale delle x dell'intervallo
	 * @return il massimo valore delle ordinate o NaN
	 */
	public double maxY(double minX, double maxX) {
		if ((x >= minX) && (x <= maxX)) {
			return y;
		} else {
			return java.lang.Double.NaN;
		}
	}
}
